import { ExponentialBackoff, TimeoutStrategy, handleType, retry, timeout, wrap } from 'cockatiel';
import { StatusCodes } from 'http-status-codes';
import { logger } from '../../utils';
import { APIError, NetworkError } from './errors';
/**
 * Default retry stats codes.
 * @public
 */
export const defaultRetryStatsCodes = [
    StatusCodes.REQUEST_TIMEOUT,
    StatusCodes.TOO_MANY_REQUESTS,
    StatusCodes.INTERNAL_SERVER_ERROR,
    StatusCodes.BAD_GATEWAY,
    StatusCodes.SERVICE_UNAVAILABLE,
    StatusCodes.GATEWAY_TIMEOUT,
];
/**
 * A fetch policy factory.
 * @remarks
 * Will retry network errors and responses with certain status codes up to the specified max attempts.
 * @param maxAttempts - Max attempts. Defaults to `2`.
 * @param retryCodes - Status code array to retry. Defaults to {@link defaultRetryStatsCodes}.
 * @param backoff - Backoff algo. Defaults to {@link cockatiel#ExponentialBackoff}.
 * @param timeoutDuration - Timeout duration. Defaults to 3 seconds.
 * @param logEvents - Log events to the console. Defaults to `true`.
 * @public
 */
export const createFetchPolicy = (maxAttempts = 2, retryCodes = defaultRetryStatsCodes, backoff = new ExponentialBackoff(), timeoutDuration = 3000, logEvents = true) => {
    const retryPolicy = retry(handleType(NetworkError).orType(APIError, (error) => retryCodes.includes(error.status)), {
        maxAttempts,
        backoff,
    });
    const timeoutPolicy = timeout(timeoutDuration, TimeoutStrategy.Aggressive);
    const policy = wrap(retryPolicy, timeoutPolicy);
    if (logEvents) {
        retryPolicy.onRetry((data) => logger.info('Fetch retrying...', data));
        retryPolicy.onFailure((data) => logger.info('Fetch failed:', data));
        retryPolicy.onSuccess((data) => logger.info('Fetch success:', data));
        retryPolicy.onGiveUp((data) => logger.info('Fetch gave up:', data));
        timeoutPolicy.onTimeout(() => logger.info('Fetch timed out'));
    }
    return policy;
};
/**
 * The default fetch policy.
 * @public
 */
export const defaultFetchPolicy = createFetchPolicy();
